/* DATECONV.C - command line utility to perform date conversion
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */

/* 
 * Usage: dateconvert [-c date] [-f field#] oldformat newformat
 * One of -f field# or -c date must be used.
 *
 * Can operate 	1) on a single date given on command line (-c)
 *		2) as a filter on a stream of data (-f), converting 
 *			dates in one field
 *
 * Available date formats are described in dmsdates(4)
 *
 * History: 5/3/00 scg created
 */
#include "tdhkit.h"
#define COMMANDLINE 1
#define FILTER 2

main( argc, argv )
int argc;
char **argv;
{
char line[ MAXRECORDLEN ];
char f[ MAXITEMS ][ DATAMAXLEN+1 ];
int i, col;
int nfields;
int stat;
char result[80];
int fld;
int mode;
char testdate[80], oldformat[80], newformat[80];
int rm_us;
int j, len;
int warnbaddate;
int topt;
int headermode;

if( argc < 5 ) {
	fprintf( stderr, "Usage: dateconvert -c date oldformat newformat [-w -u ]\n\
	(convert a date given on command line)\n" );
	fprintf( stderr, "..or.. dateconvert -f field# oldformat newformat [-w -u ]\n\
	(convert dates found in a stream of data\n" );
	exit(1);
	}

/* process config file if any.. */
TDH_readconfig( "TDHKIT_CONFIG" );
	
/* process command line args.. */
mode = COMMANDLINE;
strcpy( testdate, "" );
strcpy( oldformat, "" );
strcpy( newformat, "" );
rm_us = 0;
warnbaddate = 0;
topt = 0;
headermode = 0;

for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-c" )==0 ) {
		mode = COMMANDLINE;
		i++;
		if( i < argc ) strcpy( testdate, argv[i] );
		}
	else if( strcmp( argv[i], "-f" )==0 ) {
		mode = FILTER;
		i++;
		if( i < argc ) fld = atoi( argv[i] ) - 1;
		}
	else if( strcmp( argv[i], "-u" )==0 ) rm_us = 1;
	else if( strcmp( argv[i], "-w" )==0 ) warnbaddate = 1;
	else if( strlen( oldformat ) < 1 ) strcpy( oldformat, argv[i] );
	else if( strlen( newformat ) < 1 ) strcpy( newformat, argv[i] );
	}

stat = DT_setdatefmt( oldformat );
if( stat != 0 ) {
	fprintf( stderr, "dateconvert error: invalid oldformat (%s)\n", oldformat );
	exit(1);
	}

if( mode == COMMANDLINE ) {
	stat = DT_formatdate( testdate, newformat, result );
	if( stat != 0 ) {
		fprintf( stderr, "dateconvert error: conversion to format '%s' failed\n", newformat );
		exit(1);
		}
	if( rm_us ) {
		for( j = 0, len = strlen(result); j < len; j++ ) if( result[j] == '_' ) result[j] = ' ';
		}
	printf( "%s", result );
	exit( 0 );
	}

/* otherwise we are in FILTER mode.. read input records.. */
while( fgets( line, MAXRECORDLEN-1, stdin ) != NULL ) {
	for( i = 0; i < MAXITEMS; i++ ) strcpy( f[i], "" ); /* scg 8/6/00 */
	i = 0; col = 0;
	while( 1 ) {
		if( topt ) GL_getseg( f[ i ], line, &col, "\t\n" );
		else strcpy( f[ i ], GL_getok( line, &col ) );
		if( f[ i ][ 0 ] == '\0' ) break;
		if( i == 0 && strncmp( f[i], "//", 2 )==0 ) goto NEXT; /* skip comments */
		i++;
		}
	if( i == 0 ) continue; /* skip blank lines */
	nfields = i;

	stat = DT_formatdate( f[fld], newformat, result );
	if( stat == 0 ) {
		if( rm_us ) {
			for( j = 0, len = strlen(result); j < len; j++ ) if( result[j] == '_' ) result[j] = ' ';
			}
		strcpy( f[fld], result );
		}
	else if( warnbaddate && !headermode ) strcat( f[fld], ":BAD-DATE" ); 
	for( i = 0; i < nfields; i++ ) printf( "%s ", f[i] );
	printf( "\n" );
	headermode = 0;
	NEXT: ;
	}
exit(0);
}
