/* FLDFMT.C - command line utility to display data fields with formatting
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */

#include "tdhkit.h"
#define FS_SIZE 3000

main( argc, argv )
int argc; 
char **argv;
{
int i, j, col, stat, len;
char fmtstring[ FS_SIZE ];
char f[ MAXITEMS ][ DATAMAXLEN+ 1];
char buf[ MAXRECORDLEN ];
char outbuf[ VARSUBLINELEN ];
char tok[300];
char recordid[80];
char *s, *getenv();
int showcodes;
int hideund;
int allowinline;
int suppressdll;
int headermode;
int topt;
char fmtfile[ MAXPATH ];
int headerout;
int newfieldmode, modfieldmode;
int fieldpos;
char newcontents[ DATAMAXLEN+1];
int nfields;

TDH_errprog( "fldfmt" );
 
/* process config file if any.. */
TDH_readconfig( "TDHKIT_CONFIG" );
strcpy( recordid, "" );

/* see if recordid passed in environment.. */
s = getenv( "TDHREC" );
if( s != NULL ) strcpy( recordid, s );
 
if( argc < 2 ) {
	err( 2601, "invalid usage", "" );
	fprintf( stderr, "usage: cat data | fldfmt [options] formatstring\n" );
	exit(1);
	}

/* get arguments.. */
showcodes = 0;
hideund = 1;
allowinline = 1;
suppressdll = 0;
topt = 0;
headermode = 0;
strcpy( fmtfile, "" );
headerout = 0;
newfieldmode = modfieldmode = 0;

for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-r" )==0 ) {
		i++;
		strcpy( recordid, argv[i] );
		}
	else if( strcmp( argv[i], "-f" )==0 ) {
		i++;
		strcpy( fmtfile, argv[i] );
		}
	else if( strcmp( argv[i], "-h" )==0 ) headermode = 1;
	else if( strcmp( argv[i], "-h-" )==0 ) headermode = 0;
	else if( strcmp( argv[i], "-bh" )==0 ) headerout = 1;
	else if( strcmp( argv[i], "-t" )==0 ) topt = 1;
	else if( strcmp( argv[i], "-c" )==0 ) showcodes = 1;
	else if( strcmp( argv[i], "-u" )==0 ) hideund = 0;
	else if( strcmp( argv[i], "-l" )==0 ) allowinline = 0; /* literal mode.. don't allow inline ops.. */
	else if( strcmp( argv[i], "-s" )==0 ) suppressdll = 1; /* suppress data-less lines */
	else if( strcmp( argv[i], "-dp" )==0 ) {
		hideund = 0;
		showcodes = 1;
		}
	else if( strcmp( argv[i], "-newfield" )==0 ) {
		newfieldmode = 1;
		if( (argc-i) <= 2 ) { fprintf( stderr, "usage: fldfmt -newfield fldpos# contents\n" ); exit( 1 ); }
		fieldpos = atoi( argv[i+1] ) - 1;
		strcpy( newcontents, argv[i+2] );
		hideund = 0; showcodes = 1;
		i+=2;
		}
	else if( strcmp( argv[i], "-modfield" )==0 ) {
		modfieldmode = 1;
		if( (argc-i) <= 2 ) { fprintf( stderr, "usage: fldfmt -modfield fldpos# contents\n" ); exit( 1 ); }
		fieldpos = atoi( argv[i+1] ) - 1;
		strcpy( newcontents, argv[i+2] );
		hideund = 0; showcodes = 1;
		i+=2;
		}
	else break;
	}

if( newfieldmode || modfieldmode ) goto BEGIN;

/* now get formatstring, either from command line or file.. */
if( fmtfile[0] != '\0' ) {
	FILE *fp;
	fp = fopen( fmtfile, "r" );
	if( fp == NULL ) {
		err( 2602, "cannot open -f file", fmtfile );
		exit(1);
		}
	i = 0;
	while( fgets( buf, MAXRECORDLEN-1, fp ) != NULL ) {
		if( i + strlen( buf ) > FS_SIZE-1 ) {
			err( 2603, "too much format text", fmtfile );
			exit(1);
			}
		strcpy( &fmtstring[i], buf );
		i += strlen( buf );
		}
	}
else 	{
	strcpy( fmtstring, argv[i] );
	if( i < (argc-1) ) { /* get optional continuation arg1.. */
		i++;
		strcat( fmtstring, argv[i] );
		}
	if( i < (argc-1) ) { /* get optional continuation arg2.. */
		i++;
		strcat( fmtstring, argv[i] );
		}
	}

if( fmtstring[ strlen( fmtstring ) -1 ] != '\n' ) strcat( fmtstring, "\n" );


BEGIN:

/* value_subst settings.. */
TDH_valuesubst_settings( "hideund", hideund );
TDH_valuesubst_settings( "allowinlinecodes", allowinline );
TDH_valuesubst_settings( "suppressdll", suppressdll );

/* output bogus header */
if( headerout ) printf( "header line\n" );

/* read data and process.. */
while( fgets( buf, MAXRECORDLEN-1, stdin ) != NULL ) {
 	for( i = 0; i < MAXITEMS; i++ ) strcpy( f[i], "" ); /* scg 8/6/00 */
	if( sscanf( buf, "%s", f[0] ) < 1 ) continue; /* blank lines*/
	if( strncmp( f[0], "//", 2 )==0 ) continue;

        if( headermode ) {
                TDH_loadfieldmap( "header", buf );
                headermode = 0;
                strcpy( recordid, "header" );
                continue;
                }

	/* parse out fields.. */
        i = 0; col = 0;
        while( 1 ) {
		if( topt ) GL_getseg( f[i], buf, &col, "\t\n" );
                else strcpy( f[ i ], GL_getok( buf, &col ) );
                if( f[ i ][ 0 ] == '\0' ) break;
		if( !showcodes ) {
			if( strcmp( f[i], TDH_dbnull ) ==0 ) strcpy( f[i], "" );
			}
                i++;
                }
	nfields = i;

	/* handle -newfield and -modfield ... */
	if( newfieldmode || modfieldmode ) {
		int jj;
		if( fieldpos < 0 ) { fprintf( stderr, "invalid fieldpos#\n" ); exit( 1 ); }
		if( newfieldmode ) {
			if( fieldpos >= nfields ) jj = nfields;
			else jj = fieldpos;
			}
		else	{
			if( fieldpos >= nfields ) { fprintf( stderr, "invalid fieldpos# - not that many fields .. field can't be modified\n" ); exit( 1 ); }
			jj = fieldpos;
			}
			

		for( i = 0; i < jj; i++ ) {
			if( topt ) printf( "%s	", f[i] );
			else printf( "%s ", f[i] );
			}
		
		if( topt ) printf( "%s	", newcontents );
		else printf( "%s ", newcontents );

		if( modfieldmode ) i++;

		for( ; i < nfields; i++ ) {
			if( topt ) printf( "%s	", f[i] );
			else printf( "%s ", f[i] );
			}

		printf( "\n" );
		continue;	
		}


	/* replicate code from value_subst, so that individual lines may be suppressed.. */

	stat = TDH_value_subst( outbuf, fmtstring, f, recordid, NORMAL, 0 );
	printf( "%s", outbuf );
	}
exit( 0 );
}
