/*   usage: fldmap sourcefile destfile
 *   'sourcefile' and 'destfile' are two tabular ascii files with field name headers.
 *
 *   This program will write the contents of sourcefile, reformatted to the field 
 *   format of destfile, to stdout.  This remapping is based on field names.
 *   
 *   A warning will be issued for any field found in sourcefile that is not in 
 *   destfile, and such fields will not be present in the result.  Destfile fields
 *   not present in sourcefile will be filled in as nulls (TDH_dbnull).
 *
*/
#include "tdhkit.h"

main( argc, argv )
int argc;
char **argv;
{
int i, ix, first, stat;
char buf[ MAXRECORDLEN ];
char data[ MAXITEMS ][ DATAMAXLEN+1 ];
char sourcefile[255], destfile[255], tok[255], fieldname[50];
int mapping[ MAXITEMS ], nsrcitems, nmappedfields, ndestitems;
FILE *fp;
int doheader;

if( argc < 3 ) { 
	USAGE_ERR:
	fprintf( stderr, "usage: fldmap [-H] sourcefile destfile\n" ); 
	exit( 1 ); 
	}

TDH_readconfig( "TDHKIT_CONFIG" );

doheader = 0;
for( i = 1; i < argc; i++ ) {
	if( argv[i][0] != '-' ) break;
	if( strcmp( argv[i], "-H" )==0 ) {
		doheader = 1;
		if( argc < 4 ) goto USAGE_ERR;
		}
	}

strcpy( sourcefile, argv[i++] );
strcpy( destfile, argv[i++] );

fp = fopen( destfile, "r" );
if( fp == NULL ) { fprintf( stderr, "usage: cannot open destfile (%s) to read field name header..\n", destfile ); exit( 1 ); }

/* read dest file field names .. */
while( fgets( buf, MAXRECORDLEN-1, fp ) != NULL ) {
        if( sscanf( buf, "%s", tok ) < 1 ) continue; /* blank lines*/
        if( strncmp( tok, "//", 2 )==0 ) continue; /* comments */

        stat = TDH_loadfieldmap( "header", buf );
	if( stat != 0 ) { fprintf( stderr, "error %d on loadfieldmap\n", stat ); exit( 1 ); }

	if( doheader ) printf( "%s", buf );

	break;
        }
fclose( fp );

ndestitems = TDH_fieldmap( "header", "" );
if( ndestitems < 1 ) { fprintf( stderr, "not enough dest items\n" ); exit( 1 ); }

fp = fopen( sourcefile, "r" );
if( fp == NULL ) { fprintf( stderr, "usage: cannot open sourcefile (%s) to read field name header..\n", sourcefile ); exit( 1 ); }


/* read source file field names .. */
first = 1;
nmappedfields = 0;
while( fgets( buf, MAXRECORDLEN-1, fp ) != NULL ) {
        if( sscanf( buf, "%s", tok ) < 1 ) continue; /* blank lines*/
        if( strncmp( tok, "//", 2 )==0 ) continue; /* comments */

	if( first ) {
		ix = 0;
		for( i = 0; i < MAXITEMS; i++ ) {
			strcpy( fieldname, GL_getok( buf, &ix ));
			if( fieldname[0] == '\0' ) break;
			mapping[i] = TDH_fieldmap( "header", fieldname );
			if( mapping[i] < 0 ) 
				fprintf( stderr, "warning, field %s isn't in destfile so it won't be present in result\n", fieldname );
			else nmappedfields++;

			}


		nsrcitems = i;
		if( nmappedfields < 1 ) { fprintf( stderr, "error: no fields matched.. check field name headers..\n" ); exit( 1 ); }
		fprintf( stderr, "...mapping %d fields..\n", nmappedfields );
		first = 0;
		continue;
		}

	/* initialize output fields to null.. */
	for( i = 0; i < ndestitems; i++ ) strcpy( data[i], TDH_dbnull );

	/* parse out fields.. */
	ix = 0;
	for( i = 0; i < nsrcitems; i++ ) {
		if( mapping[i] < 0 ) { GL_getok( buf, &ix ); continue; }
		strcpy( data[ mapping[ i ] ], GL_getok( buf, &ix ));
		}

	for( i = 0; i < ndestitems; i++ ) printf( "%s ", data[i] );
	printf( "\n" );
        }
fclose( fp );

}

