/* PJOIN.C - command line utility to perform a relational join on two tabular files
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */

/* Portions written by Sandra Reynolds and Marv Newhouse marnew@jhmi.edu */

/* -i added scg 10/12/01 */


#include  "tdhkit.h"
#define YES 1
#define NO 0
#define KEYMAX 10  /* max number of keys in a key field list */
#define FROM_PIPE 1
#define FROM_FILE 2
#define FROM_STDIN 3


char line1[MAXRECORDLEN], line2[MAXRECORDLEN], image1[MAXRECORDLEN], image2[MAXRECORDLEN];
FILE *fpin1, *fpin2, *popen();
int intyp1 = 0, intyp2 = 0;
char fmt1[KEYMAX][600]; /* big enough to hold DATAMAXLEN x 4 (%*s ) */
char fmt2[KEYMAX][600];
int k1[KEYMAX];
int k2[KEYMAX];
char key1[KEYMAX][DATAMAXLEN+1];
char key2[KEYMAX][DATAMAXLEN+1];
char key_printed[KEYMAX][DATAMAXLEN+1];
int reading1, reading2;
int nk1, nk2;
char tok[DATAMAXLEN+1];
char headerline1[MAXRECORDLEN] = "", headerline2[MAXRECORDLEN] = "";
int headermode1 = 0, headermode2 = 0;
int omittedh1 = 0, omittedh2 = 0;


main( argc, argv )
int argc;
char *argv[];
{
int i, j, k, ix;
char joinsep[12];
char fillchar;
char sortcmd[200];
int diff;
int dleft, dright, dups1, dups2;
int sortq;
FILE *fp;
int headerout;

int printleftonly, printrightonly;
char kf1[255], kf2[255];
char fdf1[40], fdf2[40];
char filename1[ MAXPATH ], filename2[ MAXPATH ];
int stat;
int casesens;


TDH_errprog( "pjoin" );
stat = TDH_readconfig( "TDHKIT_CONFIG" );
/* stat = DB_readconfig( "TDHKIT_CONFIG" ); */
if( stat != 0 ) {
        err( 20, "Cannot load TDHKIT_CONFIG config file", "" );
        exit( 20 );
        }


if( argc < 3 ) { 
	err( 4300, "Usage error", "" );
	fprintf( stderr, "Usage: pjoin [options] file1 keyfields1 file2 keyfields2\n" );
	exit(1);
	}

/* set defaults */
dups1 = dups2 = 0;
dleft = dright = 0;
printleftonly = printrightonly = 0;
sortq = 1; 
strcpy( joinsep, " " ); 
fillchar = '-';
strcpy( filename1, "" );
strcpy( filename2, "" );
strcpy( kf1, "" );
strcpy( kf2, "" );
strcpy( fdf1, "" );
strcpy( fdf2, "" );
headerout = 0;
casesens = 1;

/* get options if any */
for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-l" )==0 ) dleft = 1; 	/* driven by left side */
	else if( strcmp( argv[i], "-r" )==0 ) dright = 1;	/* driven by right side */
	else if( strcmp( argv[i], "-dup1" )==0 ) dups1 = 1;	/* dups in file1 */
	else if( strcmp( argv[i], "-dup2" )==0 ) dups2 = 1;	/* dups in file2 */
	else if( strcmp( argv[i], "-t" )==0 ) strcpy( joinsep, "	" ); /* tab */
	else if( strcmp( argv[i], "-f" )==0 ) fillchar = argv[i][1];
	else if( strcmp( argv[i], "-q" )==0 ) sortq = 0; 	/* quick option - don't sort inputs */
	else if( strcmp( argv[i], "-rml" )==0 ) printrightonly = 1;  /* remove left side of join result */
	else if( strcmp( argv[i], "-rmr" )==0 ) printleftonly = 1;   /* remove right side of join result */
	else if( strcmp( argv[i], "-fdf1" )==0 ) {		/* fdf for file1 */
		if( i < argc-1 ) { i++; strcpy( fdf1, argv[i] ); }
		}
	else if( strcmp( argv[i], "-fdf2" )==0 ) {		/* fdf for file2 */
		if( i < argc-1 ) { i++; strcpy( fdf2, argv[i] ); }
		}
	else if( strcmp( argv[i], "-h1" )==0 ) headermode1 = 1; 	
	else if( strcmp( argv[i], "-h2" )==0 ) headermode2 = 1; 	
	else if( strcmp( argv[i], "-ho" )==0 ) headerout = 1;
	else if( strcmp( argv[i], "-H" )==0 ) {
		headermode1 = 1;
		headermode2 = 1;
		headerout = 1;
		}
	else if( strcmp( argv[i], "-i" )==0 ) casesens = 0;
	else break;
	}
if( dups1 && dups2 ) { err( 4301, "cant have dup keys in both files", "" ); exit( 1 ); }


/* get file1 and keyfields1 */
if( i < argc )strcpy( filename1, argv[i] );
i++;
if( i < argc )strcpy( kf1, argv[i] );
i++;

/* get file2 and keyfields2 */
if( i < argc )strcpy( filename2, argv[i] );
i++;
if( i < argc )strcpy( kf2, argv[i] );
i++;
	
/* check args.. */
if( filename1[0] == '\0' ) { err( 4302, "expecting filename1", "" ); exit( 1 ); }
if( filename2[0] == '\0' ) { err( 4302, "expecting filename2", "" ); exit( 1 ); }

if( kf1[0] == '\0' ) { err( 4302, "expecting keyfields1", "" ); exit( 1 ); }
if( kf2[0] == '\0' ) { err( 4302, "expecting keyfields2", "" ); exit( 1 ); }


/* if headermode1, read header line now (before sort).. */
if( headermode1 ) {
	fp = fopen( filename1, "r" );
	if( fp == NULL ) err( 4310, "cannot open file1", filename1 );
	while( fgets( line1, MAXRECORDLEN-1, fp ) != NULL ) {
		if( sscanf( line1, "%s", tok ) < 1 ) continue;
		if( strncmp( tok, "//", 2 )==0 ) continue;
		break;
		}
	line1[ strlen( line1 ) -1 ] = '\0';
	TDH_loadfieldmap( "header1", line1 );
	strcpy( fdf1, "header1" );
	strcpy( headerline1, line1 );
	fclose( fp );
	}


/* get keylist1 (k1) out of kf1 */
nk1 = 0; ix = 0;
while( 1 ) {
	GL_getseg( tok, kf1, &ix, "," );
	if( tok[0] == '\0' ) break;
	if( fdf1[0] != '\0' ) {
		if( GL_goodnum( tok, &k )) i = atoi( tok );
		else i = TDH_fieldmap( fdf1, tok ); /* look up name fdf */
		if( i < 0 || i > MAXITEMS ) { err( 4306, "invalid fieldname in keyfields1", tok ); exit(1); }
		else k1[ nk1 ] = i+1;
		}
	else if( atoi( tok ) > 0 ) k1[ nk1 ] = atoi( tok );
	else { err( 4303, "invalid keyfields1 specification", tok ); exit( 1 ); }
	nk1++;
	}


/* if headermode2, read header line now (before sort).. */
if( headermode2 ) {
	fpin2 = fopen( filename2, "r" );
	if( fpin2 == NULL ) err( 4310, "cannot open file2", filename2 );
	while( fgets( line2, MAXRECORDLEN-1, fpin2 ) != NULL ) {
		if( sscanf( line2, "%s", tok ) < 1 ) continue;
		if( strncmp( tok, "//", 2 )==0 ) continue;
		break;
		}
	line2[ strlen( line2 ) -1 ] = '\0';
	TDH_loadfieldmap( "header2", line2 );
	strcpy( fdf2, "header2" );
	strcpy( headerline2, line2 );
	fclose( fpin2 );
	}

/* get keylist2 (k2) out of kf2 */
nk2 = 0; ix = 0;
while( 1 ) {
	GL_getseg( tok, kf2, &ix, "," );
	if( tok[0] == '\0' ) break;
	if( fdf2[0] != '\0' ) {
		if( GL_goodnum( tok, &k )) i = atoi( tok );
		else i = TDH_fieldmap( fdf2, tok ); /* from fdf */
		if( i < 0 || i > MAXITEMS ) { err( 4306, "invalid fieldname in keyfields2", tok ); exit(1); }
		else k2[ nk2 ] = i+1;
		}
	else if( atoi( tok ) > 0 ) k2[ nk2 ] = atoi( tok );
	else { err( 4303, "invalid keyfields2 specification", tok ); exit( 1 ); }
	nk2++;
	}

if( nk1 != nk2 ) { err( 4304, "the number of keyfields must match", "" ); exit( 1 ); }


/* fprintf( stderr, "[file1=%s keys1=%s file2=%s keys2=%s]\n", filename1, kf1, filename2, kf2 ); fflush( stderr ); */

/* Open files.  If invoked with the -s option, pipe through sort. */
if( sortq == YES ) {
	strcpy( sortcmd, "sort " );
	for( i = 0; i < nk1; i++ ) {
		sprintf( tok, "-k %d,%d ", k1[i], k1[i] );
		strcat( sortcmd, tok );
		}
	strcat( sortcmd, filename1 );

	fpin1 = popen( sortcmd, "r" ); 
	if ( fpin1 == NULL ) { err( 4304, "cannot open input stream 1", sortcmd ); exit(1); }
	intyp1 = FROM_PIPE;

	strcpy( sortcmd, "sort " );
	for( i = 0; i < nk2; i++ ) {
		sprintf( tok, "-k %d,%d ", k2[i], k2[i]);
		strcat( sortcmd, tok );
		}
	strcat( sortcmd, filename2 );
	fpin2 = popen( sortcmd, "r" );
	if( fpin2 == NULL ) { err( 4304, "cannot open input stream 2", sortcmd ); exit(1); }
	intyp2 = FROM_PIPE;
	}

else 	{
	/* just file names, no sort.. */
	if( filename1[0] == '-' ) { fpin1 = stdin; intyp1 = FROM_STDIN; }
	else { fpin1 = fopen( filename1, "r" ); intyp1 = FROM_FILE; }
	if ( fpin1 == NULL ) { err( 4304, "cannot open input file 1", filename1 ); exit(1); }
	
 
	if( filename2[0] == '-' ) { fpin2 = stdin; intyp2 = FROM_STDIN; }
	else { fpin2 = fopen( filename2, "r" ); intyp1 = FROM_FILE; }
	if( fpin2 == NULL ) {  err( 4304, "cannot open input file 2", filename2 ); exit(1); }
	}


/* make formats for sscanf() to get key fields.. */
for( i = 0; i < nk1; i++ ) {
	strcpy( fmt1[i], "" );
	for( j = 0; j < k1[i] - 1; j++ ) strcat( fmt1[i], "%*s " );
	strcat( fmt1[i], "%s" ); 
	}
for( i = 0; i < nk2; i++ ) {
	strcpy( fmt2[i], "" );
	for( j = 0; j < k2[i] - 1; j++ ) strcat( fmt2[i], "%*s " );
	strcat( fmt2[i], "%s" ); 
	}

/* fprintf( stderr, "[nk1=%d nk2=%d fmt1=%s fmt2=%s]\n", nk1, nk2, fmt1, fmt2 ); fflush( stderr ); */

/* get 1st record from each file and make up record templates */
reading1 = reading2 = YES;
gln1( ); gln2( );
strcpy( image1, line1 ); strcpy( image2, line2 );
for( i = 0; i < strlen( image1 ); i++ ) 
	if( image1[i] != ' ' && image1[i] != '\t' ) image1[i] = fillchar;
for( i = 0; i < strlen( image2 ); i++ ) 
	if( image2[i] != ' ' && image2[i] != '\t' ) image2[i] = fillchar;



/* print header field on output now */
if( headerout && (headermode1 || headermode2) ) {
	if( !printrightonly ) {
		if( headermode1 ) printf( "%s ", headerline1 );
		else printf( "%s ", image1 );
		}
	if( !printleftonly ) {
		if( headermode2 ) printf( "%s ", headerline2 );
		else printf( "%s ", image2 );
		}
	printf( "\n" );
	}


/* loop */
while( reading1 || reading2 ) {

	if( ! casesens ) {
		for( i = 0; i < nk1; i++ ) {
			if( (diff = stricmp( key1[i], key2[i])) != 0 ) break;
			}
		}
	else 	{
		for( i = 0; i < nk1; i++ ) {
			if( (diff = strcmp( key1[i], key2[i])) != 0 ) break;
			}
		}

	if( diff == 0 ) {
		if( printleftonly ) printf( "%s\n", line1 );
		else if( printrightonly ) printf( "%s\n", line2 );
		else printf( "%s%s%s\n", line1, joinsep, line2 );

		/* Save this key so we know we printed a rec for it   MMN 10/96  */
		for( i = 0; i < nk1; i++ ) strcpy( key_printed[i], key1[i] );

		/* If one file does not have dups, get a record from the other which may have dups */
		if( !dups2 ) gln1();
		if( !dups1 ) gln2();
		}
	else if( diff < 0 ) { 
		/* Only print this rec if we haven't already printed one for this key  MMN 10/96  */
		if( dleft ) {
			for(i = 0; i < nk1; i++) {
				if( ( diff = strcmp( key1[i], key_printed[i] )) != 0 ) {
					if( printleftonly ) printf( "%s\n", line1 );
					else if( printrightonly ) ; /* do nothing */
					else printf( "%s%s%s\n", line1, joinsep, image2 );
					break;
					}
				}
			}
		gln1(); 
		}
	else if( diff > 0 ) { 
		/* Only print this rec if we haven't already printed one for this key  MMN 10/96  */
		if( dright ) {
			for(i = 0; i < nk2; i++) {
				if( ( diff = strcmp( key2[i], key_printed[i] )) != 0) {
					if( printleftonly ) ; /* do nothing */
					else if( printrightonly ) printf( "%s\n", line2 );
					else printf( "%s%s%s\n", image1, joinsep, line2 ); 
					break;
					}
				}
			}
		gln2(); 
		}
	}

if( intyp1 == FROM_PIPE ) pclose( fpin1 );
else if( intyp1 == FROM_FILE ) fclose( fpin1 );
if( intyp2 == FROM_PIPE ) pclose( fpin2 );
else if( intyp2 == FROM_FILE ) fclose( fpin2 );

exit( 0 );
}

/* =============================================== */

gln1()
{ 
int i;
if( reading1 ) { 
	AGAIN1:
	if( fgets( line1, MAXRECORDLEN-1, fpin1 ) == NULL ) reading1 = NO; 
	else	{
		line1[ strlen( line1 ) - 1 ] = '\0';
		/* omit the field name header when encountered - depends on sort(1) not changing line */
		if( headermode1 && !omittedh1 && strcmp( line1, headerline1 ) ==0 ) {
			omittedh1 = 1;
			goto AGAIN1;
			}
		if( sscanf( line1, "%s", tok ) < 1 ) goto AGAIN1; /* skip blank lines */
		if( strncmp( tok, "//", 2 )==0 ) goto AGAIN1; /* skip comments */
		}
	}
if( reading1 ) {
	for( i = 0; i < nk1; i++ ) sscanf( line1, fmt1[i], key1[i] );
	}
else  	{
	for( i = 0; i < nk1; i++ ) strcpy( key1[i], "~~~~~~~~~~~~" ); 
	}
return( 0 );
}

gln2()
{ 
int i;
if( reading2 ) { 
	AGAIN2:
	if( fgets( line2, MAXRECORDLEN-1, fpin2 ) == NULL ) reading2 = NO; 
	else	{
		line2[ strlen( line2 ) - 1 ] = '\0';
		/* omit the field name header when encountered - depends on sort(1) not changing line */
		if( headermode2 && !omittedh2 && strcmp( line2, headerline2 ) ==0 ) {
			omittedh2 = 1;
			goto AGAIN2;
			}
		if( sscanf( line2, "%s", tok ) < 1 ) goto AGAIN2; /* skip blank lines */
		if( strncmp( tok, "//", 2 )==0 ) goto AGAIN2; /* skip comments */
		}
	}
if( reading2 ) {
	for( i = 0; i < nk2; i++ ) sscanf( line2, fmt2[i], key2[i] );

	}
else  	{
	for( i = 0; i < nk2; i++ ) strcpy( key2[i], "~~~~~~~~~~~~" ); 
	}
return( 0 );
}

custom_function()
{ }

