/* RECSEL.C - command line utility to select records from a stream using a conditional expression
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */

#include "tdhkit.h"


main( argc, argv )
int argc; 
char **argv;
{
char line[ MAXRECORDLEN ];
char f[ MAXITEMS ][ DATAMAXLEN+1 ];
FILE *dfp;
int i, col;
char recordid[ 80 ];
int nitems;
int stat;
char buf[ VARSUBLINELEN ];
char exp[ VARSUBLINELEN ];
int negate;
int headermode;
int headerout;
char headerline[ MAXRECORDLEN ];
int suppresswarn;
char *getenv(), *s;
int topt;
 
negate = 0;
headerout = 0;
suppresswarn = 0;
headermode = 0;
topt = 0;

TDH_errprog( "recsel" );

if( argc < 2 ) {
	USAGEMSG:
	err( 2100, "invalid usage", "" );
	fprintf( stderr, "Usage: recsel [-r recordid] [-v -h -H] conditional-expression\n" );
	exit(1);
	}

/* process config file if any.. */
TDH_readconfig( "TDHKIT_CONFIG" );
strcpy( recordid, "" );



for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-r" )==0 ) {
		i++;
		if( i < argc ) strcpy( recordid, argv[i] );
		}
	else if( strcmp( argv[i], "-v" )==0 ) negate = 1;
	else if( strcmp( argv[i], "-h" )==0 ) headermode = 1;
	else if( strcmp( argv[i], "-h-" )==0 ) headermode = 0;
	else if( strcmp( argv[i], "-H" )==0 ) { headermode = 1; headerout = 1; }
	else if( strcmp( argv[i], "-t" )==0 ) topt = 1;
	else if( strcmp( argv[i], "-w" )==0 ) suppresswarn = 1;
	else break;
	}
if( i >= argc ) {
	goto USAGEMSG;
	}
strcpy( exp, argv[i] );



/* read input records.. */
while( fgets( line, MAXRECORDLEN-1, stdin ) != NULL ) {
	for( i = 0; i < MAXITEMS; i++ ) strcpy( f[i], "" ); /* scg 8/6/00 */
	i = 0; col = 0;
	while( 1 ) {
		if( topt ) GL_getseg( f[i], line, &col, "\t\n" );
		else strcpy( f[ i ], GL_getok( line, &col ) );
		if( f[ i ][ 0 ] == '\0' ) break;
		if( i == 0 && strncmp( f[i], "//", 2 )==0 ) goto BOTTOM;
		i++;
		}
	if( i == 0 ) continue; /* skip blank lines */

	if( headermode ) {
		TDH_loadfieldmap( "header", line );
		headermode = 0;
		strcpy( recordid, "header" );
		strcpy( headerline, line );
		continue;
		}
	
	stat = TDH_value_subst( buf, exp, f, recordid, FOR_CONDEX, 0 );
	if( stat == 0 && !suppresswarn ) {
		err( 2102, "error, no recognized variables were found in exp.  Use -w to suppress.", exp );
		suppresswarn = 1;
		continue;
		}
	stat = TDH_condex( buf, 0 );
	if( negate ) {
		if( stat == 0 ) stat = 1;
		else stat = 0;
		}
	if( stat == 0 ) continue;
	else if( stat == 1 ) {
		if( headerout ) {
			printf( "%s", headerline );
			headerout = 0;
			}
		printf( "%s", line );
		}
	else 	{
		err( stat, "Condex error", buf );
		exit(3);
		}
	BOTTOM: ;
	}
}
