/* XLCELLS.C - command line utility to extract "cells" eg 3C 28F from a tab-delimited file.
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */

/* Rows 1-5000 and cols A - JJ supported.
   Max cell length 255.
   Max line length 5000 chars.
   Max # of requested cells is 50.

 */
#include "tdhkit.h"
#define ROWMAXLEN 5000
#define MAXROWS 5000
#define MAXCOLS 270
#define MAXREQ 50

main( argc, argv )
int argc;
char **argv;
{
int i, j, k, len, nreq, irow, nfld;
char buf[ ROWMAXLEN ], tok[128];
char rowflag[ MAXROWS ];
int cellrow[ MAXREQ ];
int cellcol[ MAXREQ ];
char data[ MAXREQ ][ DATAMAXLEN+1 ];
char *fld[ MAXCOLS ];
char blanksym[20];

if( argc < 2 ) {
	fprintf( stderr, "usage: xlcells [-bC] cell1 .. celln\neg: xlcells 35c 35d 32d\n" );
	exit( 1 );
	}

for( i = 0; i < MAXROWS; i++ ) rowflag[i] = 0;
strcpy( blanksym, "" );

/* parse cell specs.. */
for( i = 0, k = 1; k < argc; k++ ) {

	/* options.. */
	if( strncmp( argv[k], "-b", 2 )==0 ) {
		strcpy( blanksym, &argv[k][2] );
		continue;
		}

	/* cell specs.. */
	i++;

	for( j = 0, len = strlen( argv[k] ); j < len; j++ ) if( !isdigit( argv[k][j] ) ) break;
	strncpy( tok, argv[k], j ); tok[j] = '\0';

	cellrow[i-1] = atoi( tok ) - 1;
	if( cellrow[i-1] < 0 || cellrow[i-1] > MAXROWS-1 ) {
		fprintf( stderr, "%s row out of bounds\n", argv[k] );
		exit( 1 );
		}
	else rowflag[ cellrow[i-1] ] = 1;

	len = strlen( &argv[k][j] );
	if( len == 2 ) cellcol[i-1] = (((( tolower( argv[k][j] ) - 'a' ) +1 ) * 26 ) +
		                       (( tolower( argv[k][j+1] ) - 'a' ) +1 ) ) -1;
		
	else if( len == 1 ) cellcol[i-1] = tolower( argv[k][j] ) - 'a';
	if( cellcol[i-1] < 0 || cellcol[i-1] > MAXCOLS-1 ) {
		fprintf( stderr, "%s col out of bounds\n", argv[k] );
		exit( 1 );
		}
	}
nreq = i;

/* for( i = 0; i < nreq; i++ ) printf( "[%d %d] ", cellrow[i], cellcol[i] );
 * printf( "\n" );
 * for( i = 0; i < 20; i++ ) printf( "%d %d\n", i, rowflag[i] );
 * exit(0);
 */

for( i = 0; i < MAXREQ; i++ ) data[i][0] = '\0';

irow = 0;
while( fgets( buf, ROWMAXLEN-1, stdin ) != NULL ) {

	if( rowflag[ irow ] ) {

		/* parse out fields.. */
		j = 0;
		fld[j++] = &buf[0];
		for( i = 0, len = strlen( buf ); i < len; i++ ) {
			if( buf[i] == '\t' || buf[i] == 13 || buf[i] == '\n' ) {
				buf[i] = '\0';
				fld[j++] = &buf[i+1];
				}
			}
		nfld = j-1;
			
		/* get all requests for this field and put value into data array positionally.. */
		for( i = 0; i < nreq; i++ ) {
			if( cellrow[i] == irow ) {
				if( cellcol[i] < 0 || cellcol[i] > nfld-1 ) {
					fprintf( stderr, "row %d: cell not found\n", irow+1 );
					continue;
					}
				strcpy( data[i], fld[ cellcol[i] ] );			
				}
			}
				
		}

	irow++;
	}

/* output result record.. */
for( i = 0; i < nreq; i++ ) {
	if( strlen( data[i] ) < 1 ) printf( "%s	", blanksym );
	else printf( "%s	", data[i] );
	}
printf( "\n" );
}
