/* XLCLEANER.C - command line utility to clean up tabular files, especially spreadsheet exports
 * Copyright 1998-2001 Stephen C. Grubb  (www.sgpr.net) .
 * This code is covered under the GNU General Public License (GPL);
 * see the file ./Copyright for details. */


#include "tdhkit.h"
#define ERR -1
#ifdef MAXITEMS
  #undef MAXITEMS
#endif
#define MAXITEMS 128


main( argc, argv )
int argc; char *argv[];
{
char line[ MAXRECORDLEN ];
char f[ MAXITEMS ][ DATAMAXLEN+1 ];
char buf[ VARSUBLINELEN ];
int i, j, k, col;
int nitems;
int stat;
char sepstring[20];
char commentsym[20];
int len, itemlen;
int roundingmode;
int nitemsdefined;
int linenum;
int decoffset;
int skipspacermode, skiplist[100], nskiplist;
int doublespacemode;
int padlen;
int taboutmode;
int convertblank2us;
char blankfield[20];
int f_opt;
int srcline;
char *s, *getenv();
 
/* process config file if any.. */
#ifndef STANDALONE
TDH_readconfig( "TDHKIT_CONFIG" );
#endif

sprintf( sepstring, "	\n%c", 13 ); /* tab, \n, and control-M */
strcpy( commentsym, "//" );  
roundingmode = 0;
nitemsdefined = 0;
decoffset = 0;
skipspacermode = 0;
nskiplist = 0;
doublespacemode = 0;
#ifdef STANDALONE
 taboutmode = 1;
 strcpy( blankfield, "=" );
#else
 taboutmode = 0;
 strcpy( blankfield, TDH_dbnull );
#endif
convertblank2us = 1;
padlen = 0;
f_opt = 0;


/* process command line arguments.. */
for( i = 1; i < argc; i++ ) {
	if( strcmp( argv[i], "-s" )==0 ) skipspacermode = 1;
	else if( strcmp( argv[i], "-d" )==0 ) doublespacemode = 1;
	else if( strcmp( argv[i], "-t" )==0 ) taboutmode = 1;
	else if( strcmp( argv[i], "-b" )==0 ) convertblank2us = 0;
	else if( strcmp( argv[i], "-f" )==0 ) f_opt = 1;
	else if( strcmp( argv[i], "-1" )==0 ) { /* meta option -1 */
		f_opt = 1;
		taboutmode = 1;
		roundingmode = 1;
		skipspacermode = 1;
		}
	else if( strcmp( argv[i], "-r" )==0 ) {
		roundingmode = 1;
		if( i+1 < argc && atoi( argv[i+1] ) != 0 ) {
			i++;
			decoffset = atoi( argv[i] );
			}
		}
	else if( strcmp( argv[i], "-n" )==0 ) {
		i++;
		if( i < argc ) nitemsdefined = atoi( argv[i] );
		if( nitemsdefined == 0 ) {
			fprintf( stderr, "xlcleaner -n: no number specified on command line\n" );
			exit(1);
			}
		}

	else if( strcmp( argv[i], "-c" )==0 ) {
		i++;
		if( i < argc ) strcpy( commentsym, argv[i] );
		if( strlen( commentsym ) == 0 ) {
			fprintf( stderr, "xlcleaner -c: no comment symbol specified on command line\n" );
			exit(1);
			}
		}
	else if( strcmp( argv[i], "-p" )==0 ) {
		i++;
		if( i < argc ) padlen = atoi( argv[i] );
		}
	else if( strcmp( argv[i], "-m" )==0 ) {
		i++;
		if( i < argc ) strcpy( blankfield, argv[i] );
		}
	}

if( taboutmode ) padlen = 0;


/* read input records.. */
linenum = 0;
srcline = 0;
while( fgets( line, MAXRECORDLEN-1, stdin ) != NULL ) {
	srcline++;

	if( sscanf( line, "%s", buf ) < 1 ) continue; /* skip blank line */

	/* embedded comment in input.. write out as is, preceded with a DMS comment symbol.. */
	if( commentsym[0] != '\0' && strncmp( buf, commentsym, strlen(commentsym) )==0 ) {
		if( taboutmode ) printf( "%s", line ); 
		else printf( "// %s", line );
		continue;
		}

	for( i = 0; i < MAXITEMS; i++ ) strcpy( f[i], "" ); /* scg 8/6/00 */

	i = 0; col = 0;

	len = strlen( line );


	/* fill data array.. */
	while( 1 ) {


		GL_getseg( f[i], line, &col, sepstring );

		if( f[i][0] == 13 || f[i][0] == '\n' ) break;  /* be sure last item isn't ^M or \n */


		if( skipspacermode ) {
			if( linenum == 0 && strlen( f[i] ) < 1 ) {
				fprintf( stderr, "xlcleaner info: spacer field %d being skipped\n", i+1 );
				skiplist[ nskiplist++ ] = i; /* add to skip list */
				goto SKIPFLD;
				}
			if( linenum > 0 ) {  /* see if its in skip list.. */
				for( j = 0; j < nskiplist; j++ ) if( i == skiplist[j] ) break;
				if( j < nskiplist ) {
					if( strlen( f[i] ) > 0 ) 
						fprintf( stderr, "xlcleaner warning: line %d: nonblank contents (%s) in spacer field\n",
							srcline, f[i] );
					goto SKIPFLD; /* skip it */
					}
				}
			}

		if( roundingmode ) {
			GL_autoround( f[i], decoffset );
			}
		
		/* convert embedded blanks to underscore.. */
		if( convertblank2us ) {
			itemlen = strlen( f[i] );
			for( j = 0; j < itemlen; j++ ) if( f[i][j] == ' ' ) f[i][j] = '_';
			}


		/* output the field.. */
		if( strlen( f[i] ) < 1 ) printf( "%s", blankfield );
		else printf( "%s", f[i] );

		/* eol was here (***) */

		if( taboutmode ) printf( "\t" );
		else printf( " " );

		if( col >= strlen( line ) ) break; /* end of line reached */  /* moved from above (***) - scg 6/16/03 */

		if( nitemsdefined && ( i >= (nitemsdefined-1) ) ) break;

		SKIPFLD:
		if( col >= strlen( line ) ) break; /* repeat this line for situations that jump down here.. */ 
		i++;
		}

	if( nitemsdefined && i < (nitemsdefined-1) ) {
		fprintf( stderr, "xlcleaner warning: line %d was short; had only %d fields.. padding with blanked fields to conform\n", srcline, i );
		/* pad with null fields.. */
		for( k = i+1; k < nitemsdefined-1; k++ ) printf( "%s%c", blankfield, (taboutmode)?'\t':' ' ); /* added scg 6/16/03 */
		}

	if( f_opt ) {
		if( linenum == 0 ) {
			nitemsdefined = i + 1; /* changed scg 10/13/00 */
			fprintf( stderr, "xlcleaner info: 1st record has %d fields; restricting all records to %d fields\n", 
				nitemsdefined, nitemsdefined );
			}
		else	{
			/* warn of non blank discards.. */
			while( 1 ) {
				GL_getseg( buf, line, &col, sepstring );
				if( strlen( buf ) < 1 ) break;
				else if( buf[0] == 13 || buf[0] == '\n' ) break;  /* be sure last item isn't ^M or \n */
				if( strcmp( buf, blankfield )!= 0 ) 
					fprintf( stderr, "xlcleaner warning: line %d, discarding field: %s\n", srcline, buf );
				}
			}
		}
	

	linenum++;

	/* add end-of-record padding (for DMS db-searchable results only) */
	for( i = 0; i < padlen; i++ ) printf( " " );
	printf( "\n" );
	if( doublespacemode ) printf( "     \n" );
	}
}
